SystemOrganization addCategory: #Phriak!
SystemOrganization addCategory: #'Phriak-Core'!
SystemOrganization addCategory: #'Phriak-Errors'!
SystemOrganization addCategory: #'Phriak-MapReduce'!
SystemOrganization addCategory: #'Phriak-Protocol'!
SystemOrganization addCategory: #'Phriak-Tests'!

TestCase subclass: #PhriakTestCase
	instanceVariableNames: 'client'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Tests'!

PhriakTestCase subclass: #PhriakBasicTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Tests'!

!PhriakBasicTests commentStamp: '<historical>' prior: 0!
Tests working against bucket "epigenttest" on server http://riaktest:8098/riak .

Note that if the Riak server is not reached, all tests fail, but this can take time because of timeouts. It is therefore wise to verify the connection by opening http://riaktest:8098/riak using a web browser on the client computer.!

!PhriakBasicTests methodsFor: 'tests' stamp: 'gk 11/25/2011 10:01'!
testBucketProperties
	| bucket |
	bucket := self bucketForTesting.
	self assert: ((bucket properties at: 'name') = self bucketNameForTesting)! !

!PhriakBasicTests methodsFor: 'tests' stamp: 'NicolasPetton 2/2/2012 15:16'!
testDeleteObject
	"This test used earlier 'list keys' (bucket keyNames) which is cached in Riak so it
	takes a while for the delete to be apparent. Instead we see if a get will fail (404).
	The API is apparently not so slick here... But now the tests are green."
	| bucket |
	bucket := self bucketForTesting.
	bucket newObject
		key: 'test';
		data: 'abc';
		store.
	bucket removeKey: 'test'.
	self should: [bucket at: 'test'] raise: RiakNotFoundError ! !

!PhriakBasicTests methodsFor: 'tests' stamp: 'PaulDeBruicker 9/11/2012 09:42'!
testListBucketKeys
	| bucket |
	bucket := self bucketForTesting.
	bucket newObject
		data: 'abc';
		key: 'test';
		store.
	self assert: bucket keys size equals:1! !

!PhriakBasicTests methodsFor: 'tests' stamp: 'gk 11/24/2011 23:20'!
testListBuckets
	self client buckets! !

!PhriakBasicTests methodsFor: 'tests' stamp: 'RunarJordahl 3/13/2011 20:06'!
testNoOperation! !

!PhriakBasicTests methodsFor: 'tests' stamp: 'NicolasPetton 2/2/2012 00:58'!
testPutAndGetBinaryObject
	| bucket storedObject |
	storedObject := ByteArray with: 1 with: 2 with: 3.
	bucket := self bucketForTesting.
	bucket newObject
		data: storedObject;
		key: 'test';
		contentType: 'application/binary';
		store.
	self assert: (bucket at: 'test') data = storedObject! !

!PhriakBasicTests methodsFor: 'tests' stamp: 'NicolasPetton 2/2/2012 00:59'!
testPutAndGetEmptyBinaryObject
	"Empty objects will return as nil. This does not mean that the k/v is not there. "
	| bucket storedObject |
	storedObject := ByteArray new.
	bucket := self bucketForTesting.
	bucket newObject
		data: storedObject;
		key: 'test';
		contentType: 'application/binary';
		store.
	self assert: (bucket at: 'test') data isNil! !

!PhriakBasicTests methodsFor: 'tests' stamp: 'NicolasPetton 2/2/2012 00:59'!
testPutAndGetEmptyString
	"Empty objects will return as nil. This does not mean that the k/v is not there. "
	| bucket storedObject |
	storedObject := ''.
	bucket := self bucketForTesting.
	bucket newObject
		data: storedObject;
		key: 'test';
		store.
	self assert: (bucket at: 'test') data isNil! !

!PhriakBasicTests methodsFor: 'tests' stamp: 'NicolasPetton 2/2/2012 00:59'!
testPutAndGetLargeBinaryObject
	| bucket storedObject |
	storedObject := ByteArray new: 1024 * 1024 * 2 withAll: 123.
	bucket := self bucketForTesting.
	bucket newObject
		data: storedObject;
		key: 'test';
		contentType: 'application/binary';
		store.
	self assert: (bucket at: 'test') data = storedObject! !

!PhriakBasicTests methodsFor: 'tests' stamp: 'NicolasPetton 2/2/2012 00:59'!
testPutAndGetString
	| bucket storedObject |
	storedObject := 'abc'.
	bucket := self bucketForTesting.
	bucket newObject
		data: storedObject;
		key: 'test';
		store.
	self assert: (bucket at: 'test') data = storedObject! !

PhriakTestCase subclass: #PhriakMapReduceTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Tests'!

!PhriakMapReduceTests methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 21:39'!
albumsBucket
	^self client bucketNamed: 'albums'! !

!PhriakMapReduceTests methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 21:39'!
artistsBucket
	^self client bucketNamed: 'artists'! !

!PhriakMapReduceTests methodsFor: 'running' stamp: 'NicolasPetton 2/2/2012 18:28'!
setUp
	super setUp.
	self artistsBucket removeAll.
	self albumsBucket removeAll.
	self tracksBucket removeAll! !

!PhriakMapReduceTests methodsFor: 'running' stamp: 'NicolasPetton 2/2/2012 18:28'!
tearDown
	super tearDown.
	self artistsBucket removeAll.
	self albumsBucket removeAll.
	self tracksBucket removeAll! !

!PhriakMapReduceTests methodsFor: 'tests' stamp: 'NicolasPetton 2/2/2012 18:12'!
testLinkWalking
	| artist result objects |
	
	artist := self artistsBucket newObject
		key: 'beatles';
		data: 'The Beatles';
		store;
		yourself.
		
	#('Please Please Me' 'With The Beatles' 'The Beatles' 'Let It Be') do: [:each | | album |
		album := self albumsBucket newObject
			key: each;
			data: each;
			indexesAt: 'hello' put: 'world';
			store;
			yourself.
		artist 
			linkTo: 'albums' key: each tag: 'author';
			store].
	
	artist := self artistsBucket newObject
		key: 'pink floyd';
		data: 'Pink Floyd';
		store;
		yourself.
		
	#('Atom Heart Mother' 'Have A Cigar' 'The Dark Side Of The Moon') do: [:each | | album |
		album := self albumsBucket newObject
			key: each;
			data: each;
			store;
			yourself.
		artist 
			linkTo: 'albums' key: each tag: 'author';
			store].
	
	result := self client mapReduce
		bucket: 'artists' key: 'beatles';
		link: 'albums' tag: 'author';
		mapDefault;
		run.
	
	self assert: result size = 4.
	
	objects  := result collect: [:each | 
		RiakObject fromJson: each session: self client].
	
	self assert: (objects collect: [:each | each key]) sorted equals: #('Please Please Me' 'With The Beatles' 'The Beatles' 'Let It Be') sorted! !

!PhriakMapReduceTests methodsFor: 'tests' stamp: 'NicolasPetton 2/2/2012 18:37'!
testMapReduce
	| artist result objects |
	
	"MapReduce to get the track titles of The Beatles"
	
	artist := self artistsBucket newObject
		key: 'beatles';
		data: 'The Beatles';
		store;
		yourself.
		
	#('Please Please Me' 'With The Beatles' 'The Beatles' 'Let It Be') do: [:each | | album |
		album := self albumsBucket newObject
			key: each;
			data: each;
			indexesAt: 'hello' put: 'world';
			store;
			yourself.
		artist 
			linkTo: 'albums' key: each tag: 'author';
			store].
		
	#('Two of Us' 'Dig a Pony' 'Across the Universe') do: [:each | | track |
		track := self tracksBucket newObject
			key: each;
			data: each;
			store;
			yourself.
		(self albumsBucket at: 'Let It Be') 
			linkTo: 'tracks' key: each tag: 'album';
			store].
	
	result := self client mapReduce
		bucket: 'artists' key: 'beatles';
		link: 'albums' tag: 'author';
		link: 'tracks' tag: 'album';
		map: 'function(obj) {return [obj.key];}';
		run.
	
	self assert: result sorted equals: #('Two of Us' 'Dig a Pony' 'Across the Universe') sorted! !

!PhriakMapReduceTests methodsFor: 'tests' stamp: 'NicolasPetton 2/3/2012 10:45'!
testMapReduce2
	| result expected |
	"See http://wiki.basho.com/MapReduce.html#MapReduce-via-the-HTTP-API"
	
	self bucketForTesting newObject
		key: 'p1';
		data: 'Alice was beginning to get very tired of sitting by her sister on the
bank, and of having nothing to do: once or twice she had peeped into the
book her sister was reading, but it had no pictures or conversations in
it, ''and what is the use of a book,'' thought Alice ''without pictures or
conversation?';
	store.
	
	self bucketForTesting newObject
		key: 'p2';
		data: 'So she was considering in her own mind (as well as she could, for the
hot day made her feel very sleepy and stupid), whether the pleasure
of making a daisy-chain would be worth the trouble of getting up and
picking the daisies, when suddenly a White Rabbit with pink eyes ran
close by her.';
		store.
		
	self bucketForTesting newObject
		key: 'p5';
		data: 'The rabbit-hole went straight on like a tunnel for some way, and then
dipped suddenly down, so suddenly that Alice had not a moment to think
about stopping herself before she found herself falling down a very deep
well.';
		store.
		
	result := self client mapReduce 
		bucket: self bucketNameForTesting key: 'p1';
		bucket: self bucketNameForTesting key: 'p2';
		bucket: self bucketNameForTesting key: 'p5';
		map: 'function(value) {
			var words = value.values[0].data.toLowerCase().match(/\w*/g);
			var result = [];
				for(var word in words) {
					if(words[word] !!= "") {
						var count = {};
						count[words[word]] = 1;
						result.push(count);
					}
				}
			return result;
		}';
		reduce: 'function(values) {
			var result = {};
			for (var value in values) {
				for(var word in values[value]) {
					if (word in result)
						result[word] += values[value][word];
					else
						result[word] = values[value][word];
				}
			}
			return [result];
		}';
		run.
	
	expected := Json readFrom: '{"not":1,"having":1,"down":2,"herself":2,"is":1,"sister":2,"for":2,"mind":1,"as":2,"white":1,"moment":1,"do":1,"deep":1,"a":6,"before":1,"picking":1,"about":1,"whether":1,"that":1,"sitting":1,"or":3,"very":3,"making":1,"tunnel":1,"get":1,"of":5,"the":8,"on":2,"but":1,"reading":1,"in":2,"feel":1,"her":5,"day":1,"peeped":1,"daisy":1,"rabbit":2,"nothing":1,"sleepy":1,"was":3,"chain":1,"conversation":1,"what":1,"made":1,"pictures":2,"considering":1,"thought":1,"getting":1,"pink":1,"pleasure":1,"found":1,"by":2,"eyes":1,"without":1,"up":1,"conversations":1,"went":1,"beginning":1,"hot":1,"straight":1,"into":1,"use":1,"some":1,"book":2,"then":1,"she":4,"well":2,"be":1,"alice":3,"twice":1,"could":1,"would":1,"trouble":1,"tired":1,"like":1,"hole":1,"think":1,"it":2,"falling":1,"close":1,"stopping":1,"suddenly":3,"dipped":1,"daisies":1,"with":1,"and":5,"bank":1,"way":1,"once":1,"so":2,"to":3,"when":1,"ran":1,"had":3,"worth":1,"stupid":1,"no":1,"own":1}' readStream.
	
	self assert: result first = expected! !

!PhriakMapReduceTests methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 18:27'!
tracksBucket
	^self client bucketNamed: 'tracks'! !

PhriakTestCase subclass: #PhriakRiakObjectTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Tests'!

!PhriakRiakObjectTests methodsFor: 'tests' stamp: 'NicolasPetton 2/2/2012 17:54'!
testFromJson
	| json object |
	json := '{
        "bucket": "hb",
        "key": "second",
        "vclock": "a85hYGBgzGDKBVIsLEHbN2YwJTLmsTLMPvDzCF8WAA==",
        "values": [
            {
                "metadata": {
                    "Links": [
                        ["hb","fourth","foo"]
                    ],
			  "index": {
		            "hello_bin": "world"
		        },
                    "X-Riak-VTag": "2ZKEJ2gaT57NT7xhLDPCQz",
                    "content-type": "text/plain",
                    "X-Riak-Last-Modified": "Wed, 24 Feb 2010 15:24:11 GMT",
                    "X-Riak-Meta": []
                },
                "data": "the second"
            }
        ]
    }'.

	object := RiakObject fromJson: (Json readFrom: json readStream) session: RiakSession new.
	self assert: object bucket name = 'hb'.
	self assert: object key = 'second'.
	self assert: object vclock = 'a85hYGBgzGDKBVIsLEHbN2YwJTLmsTLMPvDzCF8WAA=='.
	self assert: object links first = (RiakLink bucket: 'hb' key: 'fourth' tag: 'foo').
	self assert: object indexes first = (RiakIndex key: 'hello' value: 'world').
	self assert: object contentType = 'text/plain'.
	self assert: object data = 'the second'
	
	! !

!PhriakRiakObjectTests methodsFor: 'tests' stamp: 'NicolasPetton 2/2/2012 15:27'!
testIndexes
	| bucket index1 index2 |
	index1 := RiakIndex key: 'hello' value: 'world'.
	index2 := (RiakIndex key: 'foo' value: '201') beInteger.
	bucket := self bucketForTesting.
	bucket newObject
		key: 'test';
		data: 'abc';
		indexes: {index1. index2};
		store.
	
	self assert: ((bucket at: 'test') indexes size = 2).
	self assert: ((bucket at: 'test') indexes includes: index1).
	self assert: ((bucket at: 'test') indexes includes: index2)! !

!PhriakRiakObjectTests methodsFor: 'tests' stamp: 'NicolasPetton 2/2/2012 15:27'!
testLinks
	| bucket  |
	bucket := self bucketForTesting.
	bucket newObject
		key: 'test';
		data: 'abc';
		linkTo: 'hello' key: 'world' tag: 'foo';
		linkTo: 'foo' key: 'bar' tag: 'baz';
		store.
	
	self assert: ((bucket at: 'test') links size = 2).
	self assert: ((bucket at: 'test') links includes: 
		(RiakLink bucket: 'hello' key: 'world' tag: 'foo')).
	self assert: ((bucket at: 'test') links includes: 
		(RiakLink bucket: 'foo' key: 'bar' tag: 'baz'))! !

!PhriakRiakObjectTests methodsFor: 'tests' stamp: 'NicolasPetton 2/2/2012 15:27'!
testMetadata
	| bucket object |
	bucket := self bucketForTesting.
	object := bucket newObject
		key: 'test';
		data: 'abc';
		yourself.
	object metadata at: 'hello' put: 'world'.
	object store.
	self assert: ((bucket at: 'test') metadata at: 'hello') equals: 'world'! !

!PhriakRiakObjectTests methodsFor: 'tests' stamp: 'NicolasPetton 2/2/2012 16:48'!
testSecondaryIndex
	| bucket index |
	index := RiakIndex key: 'hello' value: 'world'.
	bucket := self bucketForTesting.
	bucket newObject
		key: 'test';
		data: 'abc';
		indexes: {index};
		store.
	RiakObject new
		bucket: bucket;
		key: 'test2';
		data: 'abc';
		indexes: {index};
		store.
	self assert: ((bucket keysIndexed: index key value: index value) includes: 'test').
	self assert: ((bucket keysIndexed: index key value: index value) includes: 'test2').
	self assert: (bucket keysIndexed: index key value: index value) size = 2! !

!PhriakRiakObjectTests methodsFor: 'tests' stamp: 'NicolasPetton 2/2/2012 16:29'!
testVclock
	| bucket object vclock |
	bucket := self bucketForTesting.
	 bucket newObject
		key: 'test';
		data: 'abc';
		store.
	object := bucket at: 'test'.
	vclock := object vclock.
	object store.
	self deny: (bucket at: 'test') vclock = vclock.
	self assert: (bucket at: 'test') vclock notNil
	! !

PhriakTestCase subclass: #PhriakSiblingsTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Tests'!

!PhriakSiblingsTests methodsFor: 'accessing' stamp: 'NicolasPetton 2/28/2012 23:49'!
bucketNameForTesting 
	^'phriak_bucket_siblings_test'! !

!PhriakSiblingsTests methodsFor: 'tests' stamp: 'NicolasPetton 2/4/2012 00:22'!
testSiblings
	| bucket siblings o1 o2 |
	
	"Set the property 'allow_mult' to true first on the bucket.
	See http://wiki.basho.com/Vector-Clocks.html"
	
	bucket := self bucketForTesting.
	bucket removeAll.
	(Delay forSeconds: 2) wait.
	
	bucket properties at: 'allow_mult' put: true.
	bucket storeProperties.
	
	bucket newObject
		key: 'test';
		data: '1';
		store.
		
	o1 := bucket at: 'test'.
	o2 := bucket at: 'test'.
	
	o1 data: '2'; store.
	o2 data: '3'; store.
	
	self should: [bucket at: 'test'] raise: RiakConflictError.
	[bucket at: 'test']
		on: RiakConflictError 
		do: [:ex |
			siblings := ex siblings.
			self assert: ex bucket equals: bucket.
			self assert: siblings size equals: 2].
		
	self assert: (bucket at: 'test' vtag: siblings first) key equals: 'test'.
	self assert: (bucket at: 'test' vtag: siblings last) key equals: 'test'.! !

!PhriakSiblingsTests methodsFor: 'tests' stamp: 'NicolasPetton 2/29/2012 00:01'!
testSiblingsWithoutAllowMult
	| bucket o1 o2 |
	
	"Set the property 'allow_mult' to true first on the bucket.
	See http://wiki.basho.com/Vector-Clocks.html"
	
	bucket := RiakSession default bucketNamed: self bucketNameForTesting, '2'.
	bucket removeAll.
	(Delay forSeconds: 2) wait.
	
	bucket properties at: 'allow_mult' put: false.
	bucket storeProperties.
	
	bucket newObject
		key: 'test';
		data: '1';
		store.
		
	o1 := bucket at: 'test'.
	o2 := bucket at: 'test'.
	
	o1 data: '2'; store.
	o2 data: '3'; store.
	
	self shouldnt: [bucket at: 'test'] raise: RiakConflictError.
	
	"Last write wins!!"
	self assert: (bucket at: 'test') data equals: '3'! !

!PhriakTestCase methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 12:28'!
bucketForTesting
	^ self client bucketNamed: self bucketNameForTesting ! !

!PhriakTestCase methodsFor: 'running' stamp: 'NicolasPetton 2/1/2012 12:28'!
bucketNameForTesting
	^ 'epigenttest'! !

!PhriakTestCase methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 12:28'!
client
	^ client! !

!PhriakTestCase methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 12:28'!
client: anObject
	client := anObject! !

!PhriakTestCase methodsFor: 'running' stamp: 'NicolasPetton 2/23/2012 13:36'!
setUp
	super setUp.
	self client: RiakSession default.
	self bucketForTesting removeAll! !

!PhriakTestCase methodsFor: 'running' stamp: 'NicolasPetton 2/1/2012 12:28'!
tearDown
	super tearDown.
	self bucketForTesting removeAll! !

Object subclass: #RiakBucket
	instanceVariableNames: 'name session properties objectClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Core'!

!RiakBucket commentStamp: 'NicolasPetton 2/2/2012 13:28' prior: 0!
Represents a "bucket" in Riak. Currently holds only the name and the session we were loaded in.

You may retrieve a bucket using "aRiakSession bucketNamed: aString"!

!RiakBucket class methodsFor: 'instance creation' stamp: 'gk 11/24/2011 22:55'!
name: aString session: aSession
	^ self new
		session: aSession;
		name: aString;
		yourself! !

!RiakBucket methodsFor: 'public' stamp: 'NicolasPetton 2/2/2012 15:58'!
at: aString
	^self protocol fetchObjectOperation 
		bucket: self;
		key: aString;
		run! !

!RiakBucket methodsFor: 'public' stamp: 'NicolasPetton 2/4/2012 22:04'!
at: aString ifAbsent: aBlock
	^[self at: aString]
		on: RiakNotFoundError 
		do: [:ex | aBlock value]! !

!RiakBucket methodsFor: 'public' stamp: 'NicolasPetton 1/17/2013 01:24'!
at: aString put: aRiakObject

	^ self protocol storeObjectOperation
		object: (aRiakObject
			bucket: self;
			yourself);
		run! !

!RiakBucket methodsFor: 'public' stamp: 'NicolasPetton 2/4/2012 00:20'!
at: aString vtag: aString2
	^self protocol fetchObjectOperation 
		bucket: self;
		key: aString;
		vtag: aString2;
		run! !

!RiakBucket methodsFor: 'public-development' stamp: 'NicolasPetton 2/2/2012 16:17'!
keys
	"Only to be used in development - expensive."

	^self protocol listKeysOperation
		bucket: self;
		run! !

!RiakBucket methodsFor: 'public' stamp: 'PaulDeBruicker 9/10/2012 17:09'!
keysIndexed: aString start: aString2 end: aString3
	^self protocol secondaryIndexOperation
		bucket: self;
		key: aString;
		start: aString2
		end: aString3;
		run! !

!RiakBucket methodsFor: 'public' stamp: 'NicolasPetton 2/2/2012 16:38'!
keysIndexed: aString value: anotherString
	^self protocol secondaryIndexOperation
		bucket: self;
		key: aString;
		value: anotherString;
		run! !

!RiakBucket methodsFor: 'accessing' stamp: 'gk 11/24/2011 22:44'!
name
	^ name! !

!RiakBucket methodsFor: 'accessing' stamp: 'gk 11/24/2011 22:48'!
name: aString
	name := aString! !

!RiakBucket methodsFor: 'accessing' stamp: 'NicolasPetton 2/23/2012 12:46'!
newObject
	"Answer a new object to be stored in the Riak bucket represented by
	the receiver"
	
	^self objectClass on: self! !

!RiakBucket methodsFor: 'accessing' stamp: 'NicolasPetton 2/23/2012 12:46'!
objectClass
	^objectClass ifNil: [RiakObject]! !

!RiakBucket methodsFor: 'accessing' stamp: 'NicolasPetton 2/23/2012 12:46'!
objectClass: aClass
	objectClass := aClass! !

!RiakBucket methodsFor: 'printing' stamp: 'NicolasPetton 2/2/2012 21:41'!
printOn: stream

	stream 
		nextPutAll: 'RiakBucket[ ';
		nextPutAll: (self session protocol baseUrl 
			addPathSegment: self session protocol bucketsPath;
			addPathSegment: self name;
			asString);
		nextPutAll: ' ]'! !

!RiakBucket methodsFor: 'public' stamp: 'NicolasPetton 2/2/2012 15:58'!
properties
	"Read the bucket properties if not set yet.
	Answer the properties dictionary"

	^properties ifNil: [
		properties := self protocol getBucketOperation
			bucket: self;
			run]! !

!RiakBucket methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 15:58'!
protocol
	^self session protocol! !

!RiakBucket methodsFor: 'public-development' stamp: 'gk 11/24/2011 22:59'!
removeAll
	"Only to be used in development - expensive."

	self keys do: [:each | self removeKey: each]! !

!RiakBucket methodsFor: 'public' stamp: 'NicolasPetton 2/2/2012 16:13'!
removeKey: key
	^ self protocol deleteKeyOperation
		bucket: self;
		key: key;
		run! !

!RiakBucket methodsFor: 'accessing' stamp: 'gk 11/24/2011 10:56'!
session
	^ session! !

!RiakBucket methodsFor: 'accessing' stamp: 'gk 11/24/2011 10:56'!
session: anObject
	session := anObject! !

!RiakBucket methodsFor: 'public' stamp: 'NicolasPetton 2/2/2012 15:59'!
storeProperties
	"Store the bucket properties.
	Fails if the properties have not been read yet!!"

	properties ifNil: [
		RiakProtocolError signal: 'The bucket [', self name, '] properties have not been read yet.'].
	
	self protocol setBucketOperation 
		bucket: self;
		run! !

Object subclass: #RiakIndex
	instanceVariableNames: 'key value binary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Core'!

!RiakIndex commentStamp: 'NicolasPetton 1/31/2012 18:30' prior: 0!
a RiakIndex represents a seconday index in Riak.
Note that in order to work, in may require some backend configuration of the Riak cluster.

See http://wiki.basho.com/Secondary-Indexes.html!

!RiakIndex class methodsFor: 'instance creation' stamp: 'NicolasPetton 2/2/2012 00:18'!
fromJson: json session: aSession
	| index key |
	key := json key allButLast: 4.
	index := self key: key value: json value.
	index binary: (json key endsWith: '_bin').
	^index! !

!RiakIndex class methodsFor: 'instance creation' stamp: 'NicolasPetton 2/1/2012 12:17'!
key: aKey value: aValue
	^self new
		key: aKey;
		value: aValue;
		yourself! !

!RiakIndex methodsFor: 'comparing' stamp: 'NicolasPetton 2/1/2012 23:31'!
= anIndex
	^anIndex class == self class and: [
		anIndex key = self key and: [
			anIndex value = self value]]! !

!RiakIndex methodsFor: 'accessing' stamp: 'NicolasPetton 1/31/2012 18:31'!
beBinary
	binary := true! !

!RiakIndex methodsFor: 'accessing' stamp: 'NicolasPetton 1/31/2012 18:51'!
beInteger
	binary := false! !

!RiakIndex methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 23:41'!
binary: aBoolean
	binary := aBoolean! !

!RiakIndex methodsFor: 'testing' stamp: 'NicolasPetton 1/31/2012 18:51'!
isBinary
	^ binary ifNil: [true]! !

!RiakIndex methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:44'!
key
	^key! !

!RiakIndex methodsFor: 'accessing' stamp: 'NicolasPetton 1/31/2012 18:49'!
key: aString
	key := aString! !

!RiakIndex methodsFor: 'accessing' stamp: 'NicolasPetton 1/31/2012 18:30'!
value
	^value! !

!RiakIndex methodsFor: 'accessing' stamp: 'NicolasPetton 1/31/2012 18:30'!
value: anObject
	value := anObject! !

Object subclass: #RiakInput
	instanceVariableNames: 'bucket key keyData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-MapReduce'!

!RiakInput commentStamp: '<historical>' prior: 0!
A RiakInput provides support for passing input to Riak in an easy way.
Also able to write itself out as JSON which is the format Riak expects for queries information.!

!RiakInput methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 18:42'!
bucket
	^ bucket! !

!RiakInput methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 18:42'!
bucket: anObject
	bucket := anObject! !

!RiakInput methodsFor: 'printing' stamp: 'PaulDeBruicker 9/11/2012 09:42'!
jsonWriteOn: aStream
	self key
		ifNil: [ self bucket jsonWriteOn: aStream ]
		ifNotNil: [ 
			(self keyData
				ifNil: [ 
					{(self bucket).
					(self key)} ]
				ifNotNil: [ 
					{(self bucket).
					(self key).
					(self keyData)} ]) jsonWriteOn: aStream ]! !

!RiakInput methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 18:42'!
key
	^ key! !

!RiakInput methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 18:42'!
key: anObject
	key := anObject! !

!RiakInput methodsFor: 'accessing' stamp: 'PaulDeBruicker 9/10/2012 19:17'!
keyData
	^ keyData! !

!RiakInput methodsFor: 'accessing' stamp: 'PaulDeBruicker 9/10/2012 19:18'!
keyData: aString
	keyData := aString! !

RiakInput subclass: #RiakSecondaryIndexInput
	instanceVariableNames: 'index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-MapReduce'!

!RiakSecondaryIndexInput methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 9/10/2012 19:18'!
index
	^ index! !

!RiakSecondaryIndexInput methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 9/10/2012 19:19'!
index: anIndex
	index := anIndex! !

!RiakSecondaryIndexInput methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 9/10/2012 19:23'!
jsonValueArray
	^ {('bucket' -> self bucket).
	('index' -> self index).
	('key' -> self key)}! !

!RiakSecondaryIndexInput methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 9/10/2012 21:19'!
jsonWriteOn: aStream
	| size |
	size := self jsonValueArray size.
	aStream nextPut: ${.
	self jsonValueArray
		doWithIndex: [ :each :idx | 
			each jsonWriteOn: aStream.
			idx = size
				ifFalse: [ aStream nextPut: $, ] ].
	aStream nextPut: $}! !

RiakSecondaryIndexInput subclass: #RiakSecondaryIndexRangeInput
	instanceVariableNames: 'start end'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-MapReduce'!

!RiakSecondaryIndexRangeInput methodsFor: 'accessing' stamp: 'PaulDeBruicker 9/10/2012 20:51'!
end
	^ end! !

!RiakSecondaryIndexRangeInput methodsFor: 'accessing' stamp: 'PaulDeBruicker 9/10/2012 20:51'!
end: anObject
	end := anObject! !

!RiakSecondaryIndexRangeInput methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 9/10/2012 20:51'!
jsonValueArray
	^ {('bucket' -> self bucket).
	('index' -> self index).
	('start' -> self start).
	('end' -> self end)}! !

!RiakSecondaryIndexRangeInput methodsFor: 'accessing' stamp: 'PaulDeBruicker 9/10/2012 20:51'!
start
	^ start! !

!RiakSecondaryIndexRangeInput methodsFor: 'accessing' stamp: 'PaulDeBruicker 9/10/2012 20:51'!
start: anObject
	start := anObject! !

Object subclass: #RiakLink
	instanceVariableNames: 'bucket key tag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Core'!

!RiakLink class methodsFor: 'instance creation' stamp: 'NicolasPetton 2/1/2012 12:17'!
bucket: aBucket key: aKey tag: aTag
	^self new
		bucket: aBucket;
		key: aKey;
		tag: aTag;
		yourself! !

!RiakLink class methodsFor: 'instance creation' stamp: 'NicolasPetton 2/2/2012 17:47'!
fromJson: json session: aSession
	^self new
		bucket: (json at: 1 ifAbsent: [nil]);
		key: (json at: 2 ifAbsent: [nil]);
		tag: (json at: 3 ifAbsent: [nil]);
		yourself! !

!RiakLink methodsFor: 'comparing' stamp: 'NicolasPetton 2/1/2012 23:31'!
= aLink
	^aLink class == self class and: [
		aLink bucket = self bucket and: [
			aLink key = self key and: [
				aLink tag = self tag]]]! !

!RiakLink methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 12:16'!
bucket
	^ bucket! !

!RiakLink methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 12:16'!
bucket: anObject
	bucket := anObject! !

!RiakLink methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 12:16'!
key
	^ key! !

!RiakLink methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 12:16'!
key: anObject
	key := anObject! !

!RiakLink methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 12:16'!
tag
	^ tag! !

!RiakLink methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 12:16'!
tag: anObject
	tag := anObject! !

Object subclass: #RiakLinkQuery
	instanceVariableNames: 'bucket tag keep'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-MapReduce'!

!RiakLinkQuery methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 17:47'!
bucket
	^ bucket! !

!RiakLinkQuery methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 17:47'!
bucket: anObject
	bucket := anObject! !

!RiakLinkQuery methodsFor: 'printing' stamp: 'NicolasPetton 2/1/2012 18:55'!
jsonWriteOn: aStream
	| query |
	query := Dictionary new.
	self bucket ifNotNil: [query at: 'bucket' put: self bucket].
	self tag ifNotNil: [query at: 'tag' put: self tag].
	self keep ifNotNil: [query at: 'keep' put: self keep].
	Dictionary new 
		at: 'link' put: query;
		jsonWriteOn: aStream! !

!RiakLinkQuery methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 17:47'!
keep
	^ keep! !

!RiakLinkQuery methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 17:47'!
keep: anObject
	keep := anObject! !

!RiakLinkQuery methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 17:47'!
tag
	^ tag! !

!RiakLinkQuery methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 17:47'!
tag: anObject
	tag := anObject! !

Object subclass: #RiakMapReduce
	instanceVariableNames: 'session inputs query'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-MapReduce'!

!RiakMapReduce class methodsFor: 'instance creation' stamp: 'NicolasPetton 2/1/2012 17:50'!
on: aSession
	^self new 
		session: aSession; 
		yourself! !

!RiakMapReduce methodsFor: 'adding' stamp: 'NicolasPetton 2/3/2012 16:22'!
bucket: aString 
	inputs := RiakInput new
		bucket: aString;
		yourself! !

!RiakMapReduce methodsFor: 'adding' stamp: 'PaulDeBruicker 9/10/2012 21:20'!
bucket: aString index: aString2 key: aString3
	"Secondary Index input.
	See http://wiki.basho.com/Secondary-Indexes.html#Exact-Match-Query"

	self
		inputs:
			(RiakSecondaryIndexInput new
				bucket: aString;
				index: aString2;
				key: aString3;
				yourself)! !

!RiakMapReduce methodsFor: 'adding' stamp: 'PaulDeBruicker 9/28/2012 12:15'!
bucket: aString index: aString2 start: aString3 stop: aString4
	"Secondary Index input.
	See http://wiki.basho.com/Secondary-Indexes.html#Exact-Match-Query"

	self
		inputs:
			(RiakSecondaryIndexRangeInput new
				bucket: aString;
				index: aString2;
				start: aString3;
				end: aString4;
				yourself)! !

!RiakMapReduce methodsFor: 'adding' stamp: 'NicolasPetton 2/1/2012 18:49'!
bucket: aString key: aString2 
	self inputs add: (RiakInput new
		bucket: aString;
		key: aString2;
		yourself)! !

!RiakMapReduce methodsFor: 'adding' stamp: 'PaulDeBruicker 9/10/2012 19:17'!
bucket: aString key: aString2 keyData: aString3
	self inputs
		add:
			(RiakInput new
				bucket: aString;
				key: aString2;
				keyData: aString3;
				yourself)! !

!RiakMapReduce methodsFor: 'defaults' stamp: 'NicolasPetton 2/3/2012 17:43'!
defaultMapQuery
	^RiakMapReduceQuery new
		type: 'map';
		source: 'function(v){return [v];}';
		language: 'javascript';
		yourself! !

!RiakMapReduce methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 17:49'!
inputs
	^inputs ifNil: [inputs := OrderedCollection new]! !

!RiakMapReduce methodsFor: 'accessing' stamp: 'PaulDeBruicker 9/10/2012 20:55'!
inputs: aRiakInput
	inputs := aRiakInput! !

!RiakMapReduce methodsFor: 'printing' stamp: 'NicolasPetton 2/1/2012 18:35'!
jsonWriteOn: aStream
	^Dictionary new
		at: 'inputs' put: self inputs;
		at: 'query' put: self query;
		jsonWriteOn: aStream! !

!RiakMapReduce methodsFor: 'adding' stamp: 'NicolasPetton 2/1/2012 19:10'!
link: aBucketString
	self link: aBucketString tag: nil keep: nil! !

!RiakMapReduce methodsFor: 'adding' stamp: 'NicolasPetton 2/1/2012 19:10'!
link: aBucketString tag: aString2
	self link: aBucketString tag: aString2 keep: nil! !

!RiakMapReduce methodsFor: 'adding' stamp: 'NicolasPetton 2/1/2012 19:10'!
link: aBucketString tag: aString keep: aBoolean
	self query add: (RiakLinkQuery new
		bucket: aBucketString;
		tag: aString;
		keep: aBoolean;
		yourself)! !

!RiakMapReduce methodsFor: 'adding' stamp: 'NicolasPetton 2/3/2012 17:42'!
map: aString
	self query add: (RiakMapReduceQuery new
		source: aString;
		language: 'javascript';
		type: 'map';
		yourself)! !

!RiakMapReduce methodsFor: 'adding' stamp: 'NicolasPetton 2/1/2012 18:56'!
mapDefault
	self query add: self defaultMapQuery! !

!RiakMapReduce methodsFor: 'adding' stamp: 'PaulDeBruicker 5/21/2012 12:05'!
name: aString
	"use one of Riaks built in JS funcitons"

	self query
		add:
			(RiakMapReduceQuery new
				name: aString;
				language: 'javascript';
				type: 'map';
				yourself)! !

!RiakMapReduce methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 18:51'!
query
	^query ifNil: [query := OrderedCollection new]! !

!RiakMapReduce methodsFor: 'adding' stamp: 'NicolasPetton 2/3/2012 17:43'!
reduce: aString
	self query add: (RiakMapReduceQuery new
		source: aString;
		language: 'javascript';
		type: 'reduce';
		yourself)! !

!RiakMapReduce methodsFor: 'running' stamp: 'NicolasPetton 2/2/2012 16:51'!
run
	^self session protocol mapReduceOperation 
		mapReduce: self;
		run! !

!RiakMapReduce methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 17:50'!
session
	^session! !

!RiakMapReduce methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 17:50'!
session: aSession
	session := aSession! !

Object subclass: #RiakMapReduceQuery
	instanceVariableNames: 'attributes type'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-MapReduce'!

!RiakMapReduceQuery methodsFor: 'accessing' stamp: 'NicolasPetton 2/3/2012 17:39'!
attributes
	^attributes ifNil: [attributes := Dictionary new]! !

!RiakMapReduceQuery methodsFor: 'accessing' stamp: 'NicolasPetton 2/3/2012 17:39'!
function
	^ self attributes at: 'function' ifAbsent: [nil]! !

!RiakMapReduceQuery methodsFor: 'accessing' stamp: 'NicolasPetton 2/3/2012 17:39'!
function: aString
	^ self attributes at: 'function' put: aString! !

!RiakMapReduceQuery methodsFor: 'as yet unclassified' stamp: 'NicolasPetton 2/3/2012 17:44'!
initialize
	super initialize.
	self language: 'javascript'! !

!RiakMapReduceQuery methodsFor: 'printing' stamp: 'NicolasPetton 2/3/2012 17:39'!
jsonWriteOn: aStream
	^Dictionary new
		at: self type 
		put: self attributes;
		jsonWriteOn: aStream! !

!RiakMapReduceQuery methodsFor: 'accessing' stamp: 'NicolasPetton 2/3/2012 17:40'!
language
	^ self attributes at: 'language' ifAbsent: [nil]! !

!RiakMapReduceQuery methodsFor: 'accessing' stamp: 'NicolasPetton 2/3/2012 17:40'!
language: aString
	^ self attributes at: 'language' put: aString! !

!RiakMapReduceQuery methodsFor: 'accessing' stamp: 'NicolasPetton 2/3/2012 17:40'!
module
	^ self attributes at: 'module' ifAbsent: [nil]! !

!RiakMapReduceQuery methodsFor: 'accessing' stamp: 'NicolasPetton 2/3/2012 17:40'!
module: aString
	^ self attributes at: 'module' put: aString! !

!RiakMapReduceQuery methodsFor: 'accessing' stamp: 'NicolasPetton 2/3/2012 20:00'!
name
	^ self attributes at: 'name' ifAbsent: [nil]! !

!RiakMapReduceQuery methodsFor: 'accessing' stamp: 'NicolasPetton 2/3/2012 20:00'!
name: aString
	^ self attributes at: 'name' put: aString! !

!RiakMapReduceQuery methodsFor: 'accessing' stamp: 'NicolasPetton 2/3/2012 17:40'!
source
	^ self attributes at: 'source' ifAbsent: [nil]! !

!RiakMapReduceQuery methodsFor: 'accessing' stamp: 'NicolasPetton 2/3/2012 17:40'!
source: aString
	^ self attributes at: 'source' put: aString! !

!RiakMapReduceQuery methodsFor: 'accessing' stamp: 'NicolasPetton 2/3/2012 17:33'!
type
	^ type ifNil: ['map']! !

!RiakMapReduceQuery methodsFor: 'accessing' stamp: 'NicolasPetton 2/3/2012 17:33'!
type: anObject
	type := anObject! !

Object subclass: #RiakObject
	instanceVariableNames: 'bucket key data contentType links indexes vclock lastModified metadata'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Core'!

!RiakObject commentStamp: 'NicolasPetton 2/2/2012 13:12' prior: 0!
RiakObject represents the data and metadata stored in a bucket/key pair in the Riak database, the base unit of data manipulation.

A RiakObject knows its cluster and is saved on #store call.
Its data can be a String or ByteArray!

!RiakObject class methodsFor: 'instance creation' stamp: 'NicolasPetton 2/28/2012 19:11'!
fromJson: json session: aSession
	| object values |
	json at: 'not_found' ifPresent: [^nil].
	object := self on: (aSession bucketNamed: (json at: 'bucket')).
	object
		key: (json at: 'key');
		vclock: (json at: 'vclock').
	values := (json at: 'values') first.
	object
		data: (values at: 'data');
		contentType: ((values at: 'metadata') at: 'content-type').
	((values at: 'metadata') at: 'X-Riak-Meta') ifNotNil: [:metadata |
		metadata keysAndValuesDo: [:key :value |
			object metadata at: (key allButFirst: 12) put: value]].
	((values at: 'metadata') at: 'index') ifNotNil: [:index |
		index keysAndValuesDo: [:key :value |
			object indexes add: (RiakIndex fromJson: key -> value session: aSession)]].
	((values at: 'metadata') at: 'Links') ifNotNil: [:links |
		object links: (links collect: [:each | 
			RiakLink fromJson: each session: aSession])].
	^object! !

!RiakObject class methodsFor: 'instance creation' stamp: 'NicolasPetton 2/1/2012 22:45'!
on: aBucket
	^self new
		bucket: aBucket;
		yourself! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 13:13'!
bucket
	"Answer the bucket object in which the receiver is contained"
	
	^bucket! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 22:42'!
bucket: aString
	bucket := aString! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 13:21'!
contentType
	"The MIME content-type of the receiver's data"
	
	^contentType ifNil: ['application/json']! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 15:25'!
contentType: aString
	contentType := aString! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 13:22'!
data
	"Riak is content-angostic. 
	The data can be a String (from JSON for example) or a binary object (ByteArray)"
	
	^data! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 15:24'!
data: anObject
	data := anObject! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 13:23'!
indexes
	"Answer the secondary indexes of the receiver.
	http://wiki.basho.com/Secondary-Indexes.html
	
	To add indexes, use #indexesAt:put:"
	
	^indexes ifNil: [indexes := OrderedCollection new]! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 15:36'!
indexes: aCollection
	indexes := aCollection! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/13/2012 18:37'!
indexesAt: aString
	self indexes select: [:each | each key = aString]! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 17:10'!
indexesAt: aString put: aString2
	self indexes add: (RiakIndex key: aString value: aString2)! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 13:23'!
key
	"Answer the key of the receiver in its bucket.
	Keys inside buckets must be unique."
	
	^key! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 15:24'!
key: aString
	key := aString! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/6/2012 16:29'!
lastModified
	^lastModified! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/6/2012 16:29'!
lastModified: aTimeStamp
	lastModified := aTimeStamp! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 13:24'!
linkTo: aString key: aString2 tag: aString3
	"Links are metadata that establish one-way relationships between objects in Riak. 
	http://wiki.basho.com/Links.html
	
	Add a link to an object inside a bucket named aString with the key aString2 and tagged as aString3"
	
	self links add: (RiakLink bucket: aString key: aString2 tag: aString3)! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 13:25'!
linkToObject: anObject tag: aString
	"Convenience method to link to another Riak object directly.
	See #linkTo:key:tag:"
	
	self 
		linkTo: anObject bucket name
		key: anObject key 
		tag: aString! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 13:25'!
links
	"Answer the links of the receiver.
	Links are metadata that establish one-way relationships between objects in Riak. 
	http://wiki.basho.com/Links.html"
	
	^links ifNil: [links := OrderedCollection new]! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 15:36'!
links: aCollection
	links := aCollection! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/4/2012 20:34'!
metadata
	"Answer user defined metadata"
	
	^metadata ifNil: [metadata := JsonObject new]! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:26'!
metadata: anObject
	metadata := anObject asDictionary! !

!RiakObject methodsFor: 'actions' stamp: 'NicolasPetton 2/2/2012 13:26'!
remove
	"Remove the receiver from its bucket."
	
	self bucket removeKey: self key! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/29/2012 20:55'!
removeIndexesAt: aString
	self indexes: (self indexes reject: [:each | each key = aString])! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/28/2012 18:04'!
removeLinkTo: aString key: aString2 tag: aString3
	"Links are metadata that establish one-way relationships between objects in Riak. 
	http://wiki.basho.com/Links.html
	
	Add a link to an object inside a bucket named aString with the key aString2 and tagged as aString3"
	
	self links 
		remove: (RiakLink bucket: aString key: aString2 tag: aString3)
		ifAbsent: []! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/28/2012 21:03'!
removeLinkToObject: anObject tag: aString
	self 
		removeLinkTo: anObject bucket name 
		key: anObject key 
		tag: aString! !

!RiakObject methodsFor: 'actions' stamp: 'NicolasPetton 2/2/2012 13:27'!
store
	"Store the recevier in its Riak bucket"
	
	self bucket at: self key put: self! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 15:38'!
vclock
	^vclock! !

!RiakObject methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 21:37'!
vclock: aString
	vclock := aString! !

Object subclass: #RiakOperation
	instanceVariableNames: 'protocol httpClient'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Protocol'!

!RiakOperation commentStamp: 'NicolasPetton 2/2/2012 14:14' prior: 0!
RiakOperation subclasses implement the HTTP API of Riak.

See http://wiki.basho.com/HTTP-API.html!

RiakOperation subclass: #RiakDeleteKeyOperation
	instanceVariableNames: 'bucket key'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Protocol'!

!RiakDeleteKeyOperation commentStamp: 'NicolasPetton 2/2/2012 16:12' prior: 0!
RiakDeleteObjectOperation represents a detelete object HTTP request.
It is not performed on an instance of RiakObject, but on a RiakBucket with a given key.

See http://wiki.basho.com/HTTP-Delete-Object.html!

!RiakDeleteKeyOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:12'!
bucket
	^ bucket! !

!RiakDeleteKeyOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:12'!
bucket: anObject
	bucket := anObject! !

!RiakDeleteKeyOperation methodsFor: 'executing' stamp: 'NicolasPetton 2/2/2012 16:10'!
doRun
	^self httpClient delete: self url! !

!RiakDeleteKeyOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:12'!
key
	^ key! !

!RiakDeleteKeyOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:12'!
key: anObject
	key := anObject! !

!RiakDeleteKeyOperation methodsFor: 'private' stamp: 'NicolasPetton 2/2/2012 16:12'!
successfulResponseCodes 
	"Allow 404 code. Not found responses silently succeed"
	^#(204 404)! !

!RiakDeleteKeyOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:14'!
url
	^super url
		addPathSegment: self protocol bucketsPath;
		addPathSegment: self bucket name;
		addPathSegment: 'keys';
		addPathSegment: self key! !

RiakOperation subclass: #RiakFetchObjectOperation
	instanceVariableNames: 'bucket key vtag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Protocol'!

!RiakFetchObjectOperation commentStamp: 'NicolasPetton 2/4/2012 00:15' prior: 0!
RiakFetchObjectOperation represents a fetch object HTTP request.

In case of siblings, throws a RiakConflictError error.

See http://wiki.basho.com/HTTP-Fetch-Object.html and
http://wiki.basho.com/Vector-Clocks.html about siblings and vector clocks!

!RiakFetchObjectOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 15:08'!
bucket
	^ bucket! !

!RiakFetchObjectOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 15:08'!
bucket: anObject
	bucket := anObject! !

!RiakFetchObjectOperation methodsFor: 'private' stamp: 'NicolasPetton 2/3/2012 23:37'!
errorResponseCodes
	^#(404 300)! !

!RiakFetchObjectOperation methodsFor: 'error handling' stamp: 'NicolasPetton 2/4/2012 00:10'!
handleErrorResponse: aResponse

	"The object was not found."
	aResponse statusLine code = 404 ifTrue: [RiakNotFoundError signal: self key].
	
	"Uh oh...We've go siblings!!"
	aResponse statusLine code = 300 ifTrue: [
		RiakConflictError new
			siblings: (self parseSiblingsFrom: aResponse);
			bucket: self bucket;
			signal]! !

!RiakFetchObjectOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 15:08'!
key
	^ key! !

!RiakFetchObjectOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 15:08'!
key: anObject
	key := anObject! !

!RiakFetchObjectOperation methodsFor: 'private' stamp: 'NicolasPetton 2/1/2012 23:40'!
parseIndexFrom: anAssociation
	|  k index binary |
	"Skip the 'X-Riak-Index-' part"
	k := anAssociation key asLowercase allButFirst: 13.
	binary := k endsWith: '_bin'.
	"Skip the '_bin' or '_int'"
	k := k allButLast: 4.
	index := RiakIndex key: k value: anAssociation value.
	index binary: binary.
	^index! !

!RiakFetchObjectOperation methodsFor: 'private' stamp: 'NicolasPetton 2/1/2012 23:35'!
parseIndexesFrom: aResponse
	| indexes |
	indexes := OrderedCollection new.
	aResponse headersDo: [:k :v |
		(k beginsWith: 'X-Riak-Index-') ifTrue: [
			indexes add: (self parseIndexFrom: k -> v)]].
	^indexes! !

!RiakFetchObjectOperation methodsFor: 'private' stamp: 'NicolasPetton 2/6/2012 16:28'!
parseLastModifiedFrom: aResponse
	
	"Example: 
	Last-Modified: Wed, 10 Mar 2010 18:00:04 GMT"
	
	^aResponse headers headers 
		at: 'Last-Modified' 
		ifPresent: [:string | TimeStamp fromString: (string allButFirst: 4)]
		ifAbsent: [nil]! !

!RiakFetchObjectOperation methodsFor: 'private' stamp: 'NicolasPetton 2/2/2012 16:24'!
parseLinkFrom: aString
	| path b k tag |
	"Links are of the following form: 
	</buckets/BUCKET/keys/KEY>; riaktag=''TAG'' "
	aString regex: '<[^>]+>' matchesDo: [:each |
		path := each allButFirst allButLast findTokens: $/].
	b := path second.
	k := path fourth.
	aString regex: 'riaktag="[^"]+"' matchesDo: [:each | 
		tag := each copyFrom: 10 to: each size - 1].
	^RiakLink bucket: b key: k tag: tag! !

!RiakFetchObjectOperation methodsFor: 'private' stamp: 'NicolasPetton 11/13/2012 14:18'!
parseLinksFrom: aResponse
	| links |
	links := aResponse headers at: 'Link' ifAbsent: [^nil].
	^((links findTokens: ',')
		select: [:each | each includesSubstring: 'riaktag'])
		collect: [:each | self parseLinkFrom: each]! !

!RiakFetchObjectOperation methodsFor: 'private' stamp: 'NicolasPetton 2/2/2012 16:27'!
parseMetadataFrom: anAssociation
	|  k |
	"Skip the 'X-Riak-Meta-' part"
	k := anAssociation key asLowercase allButFirst: 12.
	^Association key: k value: anAssociation value! !

!RiakFetchObjectOperation methodsFor: 'private' stamp: 'NicolasPetton 2/2/2012 16:26'!
parseMetadatasFrom: aResponse
	| metadata |
	metadata := OrderedCollection new.
	aResponse headersDo: [:k :v |
		(k beginsWith: 'X-Riak-Meta-') ifTrue: [
			metadata add: (self parseMetadataFrom: k -> v)]].
	^metadata asDictionary! !

!RiakFetchObjectOperation methodsFor: 'private' stamp: 'NicolasPetton 2/6/2012 16:28'!
parseResponse: aResponse
	^self bucket newObject
		contentType: (aResponse headers at: 'Content-Type');
		links: (self parseLinksFrom: aResponse);
		indexes: (self parseIndexesFrom: aResponse);
		metadata: (self parseMetadatasFrom: aResponse);
		key: self key; 
		data: (aResponse entity ifNotNil: [:entity | entity contents]);
		vclock: (aResponse headers at: 'X-Riak-Vclock');
		lastModified: (self parseLastModifiedFrom: aResponse);
		yourself! !

!RiakFetchObjectOperation methodsFor: 'private' stamp: 'NicolasPetton 2/4/2012 00:00'!
parseSiblingsFrom: aResponse
	"This is quite hackish. Unfortunately, Zinc doesn't parse multipart/mixed entities yet, so we can't get all siblings at once.
	
	The response body is like the following:

	Siblings:
	16vic4eU9ny46o4KPiDz1f
	4v5xOg4bVwUYZdMkqf0d6I
	6nr5tDTmhxnwuAFJDd2s6G
	6zRSZFUJlHXZ15o9CG0BYl
	
	See: http://wiki.basho.com/HTTP-Fetch-Object.html#Siblings-examples for more
	about siblings"
	
	^((aResponse entity contents copyWithRegex: 'Siblings\:' matchesReplacedWith: '')
		findTokens: Character lf)
			select: [:each | each notEmpty]! !

!RiakFetchObjectOperation methodsFor: 'private' stamp: 'NicolasPetton 2/2/2012 15:31'!
readResponse: aResponse
	^[self parseResponse: aResponse]
		on: Error
		do: [:ex | RiakParseError signal]! !

!RiakFetchObjectOperation methodsFor: 'private' stamp: 'NicolasPetton 2/2/2012 15:11'!
successfulResponseCodes
	"We do not handle multiple choices yet.
	http://wiki.basho.com/HTTP-Fetch-Object.html"
	
	^#(200 "300")! !

!RiakFetchObjectOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/4/2012 00:20'!
url
	| url |
	url := super url
		addPathSegment: self protocol bucketsPath;
		addPathSegment: self bucket name;
		addPathSegment: 'keys';
		addPathSegment: self key;
		yourself.
	self vtag ifNotNil: [url queryAt: 'vtag' put: self vtag].
	^url! !

!RiakFetchObjectOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/4/2012 00:19'!
vtag
	^ vtag! !

!RiakFetchObjectOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/4/2012 00:19'!
vtag: anObject
	vtag := anObject! !

RiakOperation subclass: #RiakGetBucketOperation
	instanceVariableNames: 'bucket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Protocol'!

!RiakGetBucketOperation commentStamp: 'NicolasPetton 2/2/2012 14:29' prior: 0!
RiakGetBucketOperation represents a get bucket properties request.

See http://wiki.basho.com/HTTP-Get-Bucket-Properties.html!

!RiakGetBucketOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 14:28'!
bucket
	^ bucket! !

!RiakGetBucketOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 14:28'!
bucket: anObject
	bucket := anObject! !

!RiakGetBucketOperation methodsFor: 'private' stamp: 'NicolasPetton 2/2/2012 14:29'!
readResponse: aResponse
	^(Json readFrom: aResponse contents readStream) at: 'props'! !

!RiakGetBucketOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/7/2012 17:57'!
url
	^super url
		addPathSegment: self protocol bucketsPath;
		addPathSegment: self bucket name;
		addPathSegment: 'props';
		yourself! !

RiakOperation subclass: #RiakListBucketsOperation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Protocol'!

!RiakListBucketsOperation commentStamp: 'NicolasPetton 2/2/2012 14:22' prior: 0!
RiakListBucketsOperation represents a bucket listing HTTP request.
List all known buckets.

*Not for production use!!*

See http://wiki.basho.com/HTTP-List-Buckets.html!

!RiakListBucketsOperation methodsFor: 'private' stamp: 'NicolasPetton 2/2/2012 14:19'!
readResponse: aResponse
	^(Json readFrom: aResponse contents readStream) at: 'buckets'! !

!RiakListBucketsOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 14:18'!
url
	^super url
		addPathSegment: self protocol bucketsPath;
		queryAt: 'buckets' put: 'true';
		yourself! !

RiakOperation subclass: #RiakListKeysOperation
	instanceVariableNames: 'bucket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Protocol'!

!RiakListKeysOperation commentStamp: 'NicolasPetton 2/2/2012 14:27' prior: 0!
RiakListKeysOperation represents a key slisting HTTP request.
List all keys in a RiakBucket.

Does not support streaming yet.

*Not for production use!!*

See http://wiki.basho.com/HTTP-List-Keys.html!

!RiakListKeysOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 14:23'!
bucket
	^ bucket! !

!RiakListKeysOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 14:23'!
bucket: anObject
	bucket := anObject! !

!RiakListKeysOperation methodsFor: 'private' stamp: 'NicolasPetton 2/2/2012 14:26'!
readResponse: aResponse
	^(Json readFrom: aResponse contents readStream) at: 'keys'! !

!RiakListKeysOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:18'!
url
	^super url
		addPathSegment: self protocol bucketsPath;
		addPathSegment: self bucket name;
		addPathSegment: 'keys';
		queryAt: 'keys' put: 'true';
		yourself! !

RiakOperation subclass: #RiakMapReduceOperation
	instanceVariableNames: 'mapReduce'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Protocol'!

!RiakMapReduceOperation methodsFor: 'executing' stamp: 'NicolasPetton 2/7/2012 18:07'!
doRun
	^self httpClient 
		url: self url;
		headerAddAll: (ZnHeaders withAll: {'Content-Type' -> 'application/json'} asDictionary);
		entity: (ZnEntity with: self mapReduce asJsonString type: 'application/json');
		post! !

!RiakMapReduceOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 18:06'!
mapReduce
	^ mapReduce! !

!RiakMapReduceOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 18:06'!
mapReduce: anObject
	mapReduce := anObject! !

!RiakMapReduceOperation methodsFor: 'private' stamp: 'NicolasPetton 2/28/2012 19:34'!
readResponse: aResponse
	^Json readFrom: aResponse entity contents readStream! !

!RiakMapReduceOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:50'!
url
	^super url
		addPathSegment: self protocol mapreducePath;
		yourself! !

!RiakOperation class methodsFor: 'instance creation' stamp: 'NicolasPetton 2/2/2012 14:13'!
on: aProtocol
	^self new 
		protocol: aProtocol; 
		yourself! !

!RiakOperation methodsFor: 'running' stamp: 'NicolasPetton 2/2/2012 14:25'!
doRun
	"Execute a request"
	^self httpClient get: self url! !

!RiakOperation methodsFor: 'private' stamp: 'NicolasPetton 2/2/2012 15:51'!
errorResponseCodes
	"Expected error response codes, handled in #run"
	^#()! !

!RiakOperation methodsFor: 'error handling' stamp: 'NicolasPetton 2/3/2012 23:57'!
handleErrorResponse: aResponse
	"Handle expected error responses.
	see #errorResponseCodes"
	
	RiakProtocolError signal: aResponse entity contents! !

!RiakOperation methodsFor: 'accessing' stamp: 'PaulDeBruicker 4/20/2012 15:57'!
httpClient
	^ httpClient ifNil: [ httpClient := ZnClient new ]! !

!RiakOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 14:15'!
protocol
	^protocol! !

!RiakOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 14:13'!
protocol: aProtocol
	protocol := aProtocol! !

!RiakOperation methodsFor: 'private' stamp: 'NicolasPetton 1/31/2012 17:33'!
readResponse: aResponse
	^aResponse entity ifNotNil: [:entity | entity contents]! !

!RiakOperation methodsFor: 'error handling' stamp: 'NicolasPetton 1/31/2012 17:10'!
responseError: aString
	RiakResponseError signal: aString! !

!RiakOperation methodsFor: 'running' stamp: 'PaulDeBruicker 9/10/2012 13:43'!
run
	
	self doRun.
	(self successfulResponseCodes includes: self httpClient response statusLine code)
		ifFalse: [ 
			(self errorResponseCodes includes: self httpClient response statusLine code)
				ifTrue: [ ^ self handleErrorResponse: self httpClient response ].
			self responseError: self httpClient response entity contents ].
	^ self readResponse: self httpClient response! !

!RiakOperation methodsFor: 'private' stamp: 'NicolasPetton 2/2/2012 14:16'!
successfulResponseCodes
	^#(200)! !

!RiakOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 14:18'!
url
	^self protocol baseUrl! !

RiakOperation subclass: #RiakSecondaryIndexOperation
	instanceVariableNames: 'bucket key start end value binary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Protocol'!

!RiakSecondaryIndexOperation commentStamp: 'NicolasPetton 2/2/2012 16:32' prior: 0!
RiakSecondaryIndexOperation represents a secondary indexes HTTP request.

It can either request an exact value, or a range (from start to end).

See http://wiki.basho.com/HTTP-Secondary-Indexes.html!

!RiakSecondaryIndexOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:41'!
binary
	^binary ifNil: [true]! !

!RiakSecondaryIndexOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:41'!
binary: aBoolean
	binary := aBoolean! !

!RiakSecondaryIndexOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:37'!
bucket
	^ bucket! !

!RiakSecondaryIndexOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:37'!
bucket: anObject
	bucket := anObject! !

!RiakSecondaryIndexOperation methodsFor: 'accessing' stamp: 'PaulDeBruicker 9/10/2012 17:09'!
end
	^ end! !

!RiakSecondaryIndexOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:38'!
key
	^key! !

!RiakSecondaryIndexOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:38'!
key: aString
	key := aString! !

!RiakSecondaryIndexOperation methodsFor: 'private' stamp: 'NicolasPetton 2/2/2012 16:36'!
readResponse: aResponse
	^(Json readFrom: aResponse contents readStream) at: 'keys'! !

!RiakSecondaryIndexOperation methodsFor: 'accessing' stamp: 'PaulDeBruicker 9/10/2012 17:09'!
start
	^ start! !

!RiakSecondaryIndexOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:35'!
start: aString end: aString2
	value := nil.
	start := aString.
	end := aString2! !

!RiakSecondaryIndexOperation methodsFor: 'accessing' stamp: 'PaulDeBruicker 9/10/2012 17:10'!
url
	| url |
	url := super url
		addPathSegment: self protocol bucketsPath;
		addPathSegment: self bucket name;
		addPathSegment: 'index';
		addPathSegment: self key, (self binary 
			ifTrue: ['_bin'] ifFalse: ['_int']);
		yourself.
	self value 
		ifNil: [url addPathSegment: self start; addPathSegment: self end]
		ifNotNil: [url addPathSegment: self  value].
	^url! !

!RiakSecondaryIndexOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:34'!
value
	^value! !

!RiakSecondaryIndexOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:34'!
value: aString
	start := end := nil.
	value := aString! !

RiakOperation subclass: #RiakSetBucketOperation
	instanceVariableNames: 'bucket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Protocol'!

!RiakSetBucketOperation commentStamp: 'NicolasPetton 2/2/2012 16:05' prior: 0!
RiakSetBucketOperation a set bucket properties HTTP request.

Properties are read from the bucket.

See http://wiki.basho.com/HTTP-Set-Bucket-Properties.html!

!RiakSetBucketOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 14:30'!
bucket
	^ bucket! !

!RiakSetBucketOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 14:30'!
bucket: anObject
	bucket := anObject! !

!RiakSetBucketOperation methodsFor: 'running' stamp: 'NicolasPetton 2/7/2012 18:06'!
doRun
	^self httpClient
		url: self url;
		headerAddAll: (ZnHeaders withAll: {'Content-Type' -> 'application/json'} asDictionary);
		entity: (ZnEntity 
			with: (Dictionary new 
				at: 'props' 
				put: self bucket properties; yourself) asJsonString
			type: 'application/json');
		put! !

!RiakSetBucketOperation methodsFor: 'private' stamp: 'NicolasPetton 2/2/2012 14:44'!
readResponse: aResponse
	"No op"! !

!RiakSetBucketOperation methodsFor: 'private' stamp: 'NicolasPetton 2/2/2012 14:32'!
successfulResponseCodes 
	^{204}! !

!RiakSetBucketOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:15'!
url
	^super url
		addPathSegment: self protocol bucketsPath;
		addPathSegment: self bucket name;
		addPathSegment: 'props';
		yourself! !

RiakOperation subclass: #RiakStoreObjectOperation
	instanceVariableNames: 'object'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Protocol'!

!RiakStoreObjectOperation commentStamp: 'NicolasPetton 2/2/2012 16:08' prior: 0!
RiakStoreObjectOperation represents a store HTTP request: 
Stores an object under the specified bucket / key. 

Use the PUT method, never POST.

Multiple choices and conditional request semantics are not handled yet.

See http://wiki.basho.com/HTTP-Store-Object.html!

!RiakStoreObjectOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:46'!
addIndexHeadersTo: header
	header addAll: (self object indexes collect: [:each || indexKey |
		indexKey := each isBinary ifTrue: [each key, '_bin'] ifFalse: [each key, '_int'].
		('x-riak-index-', indexKey) -> each value]) asDictionary! !

!RiakStoreObjectOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:23'!
addLinksHeaderTo: headers
	self object links isEmpty ifFalse: [
		headers at: 'Link' put: (String streamContents: [:str |
			self object links 
				do: [:each | str nextPutAll: '</', self protocol bucketsPath,'/', each bucket, '/keys/', each key, '>; riaktag="', each tag, '"']
				separatedBy: [str nextPut: $,]])]! !

!RiakStoreObjectOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:08'!
addMetadataHeadersTo: headers
	self object metadata keysAndValuesDo: [:key :value |
		headers at: 'X-Riak-Meta-', key put: value]! !

!RiakStoreObjectOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:07'!
addVClockHeaderTo: headers
	self object vclock ifNotNil: [
		headers at: 'X-Riak-Vclock' put: self object vclock]! !

!RiakStoreObjectOperation methodsFor: 'executing' stamp: 'PaulDeBruicker 9/10/2012 14:31'!
doRun

	^ self httpClient
		headerAddAll: self headers;
		url: self url;
		entity: (ZnEntity with: self object data type: self object contentType);
		put! !

!RiakStoreObjectOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:04'!
headers
	| headers |
	headers := ZnHeaders new.
	self 
		addIndexHeadersTo: headers;
		addLinksHeaderTo: headers;
		addVClockHeaderTo: headers;
		addMetadataHeadersTo: headers.
	^headers! !

!RiakStoreObjectOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/1/2012 16:39'!
object
	^ object! !

!RiakStoreObjectOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 19:17'!
object: anObject
	object := anObject! !

!RiakStoreObjectOperation methodsFor: 'private' stamp: 'NicolasPetton 2/2/2012 16:09'!
successfulResponseCodes 
	^#(200 201 204)! !

!RiakStoreObjectOperation methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:15'!
url
	^super url
		addPathSegment: self protocol bucketsPath;
		addPathSegment: self object bucket name;
		addPathSegment: 'keys';
		addPathSegment: self object key;
		yourself! !

Object subclass: #RiakProtocol
	instanceVariableNames: 'session host port riakPath bucketsPath mapreducePath'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Protocol'!

!RiakProtocol commentStamp: 'gk 11/24/2011 10:49' prior: 0!
Implements direct, low level communication with the Riak server using its REST API ( https://wiki.basho.com/REST-API.html ).
If you wish to use a different HTTP implementation, just override or change methods in category http-private.!

!RiakProtocol class methodsFor: 'instance creation' stamp: 'NicolasPetton 2/2/2012 16:52'!
on: aSession
	^self new 
		session: aSession;
		yourself! !

!RiakProtocol methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 14:09'!
baseUrl
	^ZnUrl new 
		scheme: 'http';
		host: self host;
		port: self port! !

!RiakProtocol methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 14:08'!
bucketsPath
	^ bucketsPath ifNil: ['buckets']! !

!RiakProtocol methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 14:08'!
bucketsPath: anObject
	bucketsPath := anObject! !

!RiakProtocol methodsFor: 'factory' stamp: 'NicolasPetton 2/2/2012 16:13'!
deleteKeyOperation
	^RiakDeleteKeyOperation on: self! !

!RiakProtocol methodsFor: 'factory' stamp: 'NicolasPetton 2/2/2012 14:48'!
fetchObjectOperation
	^RiakFetchObjectOperation on: self! !

!RiakProtocol methodsFor: 'factory' stamp: 'NicolasPetton 2/2/2012 14:35'!
getBucketOperation
	^RiakGetBucketOperation on: self! !

!RiakProtocol methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 14:08'!
host
	^ host ifNil: ['localhost']! !

!RiakProtocol methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 14:08'!
host: anObject
	host := anObject! !

!RiakProtocol methodsFor: 'factory' stamp: 'NicolasPetton 2/2/2012 14:20'!
listBucketOperation
	^RiakListBucketsOperation on: self! !

!RiakProtocol methodsFor: 'factory' stamp: 'NicolasPetton 2/2/2012 15:57'!
listKeysOperation
	^RiakListKeysOperation on: self! !

!RiakProtocol methodsFor: 'factory' stamp: 'NicolasPetton 2/2/2012 16:50'!
mapReduceOperation
	^RiakMapReduceOperation on: self! !

!RiakProtocol methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 14:08'!
mapreducePath
	^ mapreducePath ifNil: ['mapred']! !

!RiakProtocol methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 14:08'!
mapreducePath: anObject
	mapreducePath := anObject! !

!RiakProtocol methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 14:08'!
port
	^ port ifNil: [8098]! !

!RiakProtocol methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 14:08'!
port: anObject
	port := anObject! !

!RiakProtocol methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 14:08'!
riakPath
	^ riakPath ifNil: ['riak']! !

!RiakProtocol methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 14:08'!
riakPath: anObject
	riakPath := anObject! !

!RiakProtocol methodsFor: 'factory' stamp: 'NicolasPetton 2/2/2012 16:42'!
secondaryIndexOperation 
	^RiakSecondaryIndexOperation on: self! !

!RiakProtocol methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:52'!
session
	^session! !

!RiakProtocol methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:52'!
session: aSession
	session := aSession! !

!RiakProtocol methodsFor: 'factory' stamp: 'NicolasPetton 2/2/2012 14:35'!
setBucketOperation
	^RiakSetBucketOperation on: self! !

!RiakProtocol methodsFor: 'factory' stamp: 'NicolasPetton 2/2/2012 15:56'!
storeObjectOperation
	^RiakStoreObjectOperation on: self! !

Object subclass: #RiakSession
	instanceVariableNames: 'protocol'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Core'!

RiakSession class
	instanceVariableNames: 'default'!

!RiakSession commentStamp: 'gk 11/24/2011 10:49' prior: 0!
Client interface to Riak. Use this class as the starting point to interface with Riak.

Riak is Basho's "(...) Dynamo-inspired key/value store with a distributed database network platform (...)"
http://www.basho.com/products/riak/overview.php 

Holds on to a RiakProtocol which implements the direct, low level communication with the Riak server.!

RiakSession class
	instanceVariableNames: 'default'!

!RiakSession class methodsFor: 'accessing' stamp: 'NicolasPetton 2/6/2012 16:49'!
default
	^default ifNil: [default := self new]! !

!RiakSession class methodsFor: 'instance creation' stamp: 'NicolasPetton 2/2/2012 14:10'!
on: aProtocol
	^self new 
		protocol: aProtocol;
		yourself! !

!RiakSession class methodsFor: 'accessing' stamp: 'PaulDeBruicker 9/10/2012 13:18'!
resetDefault
	default := nil! !

!RiakSession methodsFor: 'factory' stamp: 'gk 11/24/2011 22:56'!
bucketNamed: aString
	^ RiakBucket name: aString session: self! !

!RiakSession methodsFor: 'accessing' stamp: 'NicolasPetton 2/3/2012 20:27'!
bucketNames
	"Only to be used in development - expensive."
	^self protocol listBucketOperation run! !

!RiakSession methodsFor: 'accessing' stamp: 'NicolasPetton 2/3/2012 20:28'!
buckets
	"Only to be used in development - expensive."
	^self bucketNames collect: [:each |
		self bucketNamed: each]! !

!RiakSession methodsFor: 'factory' stamp: 'NicolasPetton 2/2/2012 01:00'!
mapReduce
	^RiakMapReduce on: self! !

!RiakSession methodsFor: 'accessing' stamp: 'NicolasPetton 2/2/2012 16:52'!
protocol
	^protocol ifNil: [protocol := RiakProtocol on: self]! !

!RiakSession methodsFor: 'accessing' stamp: 'PaulDeBruicker 5/4/2012 09:38'!
protocol: aRiakProtocol 
	
	protocol := aRiakProtocol.
	protocol session: self ! !

Error subclass: #RiakError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Errors'!

RiakError subclass: #RiakConflictError
	instanceVariableNames: 'siblings bucket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Errors'!

!RiakConflictError methodsFor: 'accessing' stamp: 'NicolasPetton 2/3/2012 23:44'!
bucket
	^ bucket! !

!RiakConflictError methodsFor: 'accessing' stamp: 'NicolasPetton 2/3/2012 23:44'!
bucket: anObject
	bucket := anObject! !

!RiakConflictError methodsFor: 'accessing' stamp: 'NicolasPetton 2/3/2012 23:44'!
siblings
	^ siblings! !

!RiakConflictError methodsFor: 'accessing' stamp: 'NicolasPetton 2/3/2012 23:44'!
siblings: anObject
	siblings := anObject! !

RiakError subclass: #RiakNotFoundError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Errors'!

RiakError subclass: #RiakParseError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Errors'!

RiakError subclass: #RiakProtocolError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Errors'!

RiakError subclass: #RiakResponseError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phriak-Errors'!
